﻿using Syncfusion.Pdf.Parsing;
using Syncfusion.Pdf.Security;
using System.Security.Cryptography;
using System.Security.Cryptography.Pkcs;
using System.Security.Cryptography.X509Certificates;

namespace Sample
{
    class Program
    {
        static void Main(string[] args)
        {
            PdfLoadedDocument document = new PdfLoadedDocument("../../Barcode.pdf");

            //Create a revision 2 signature with loaded digital ID.
            PdfSignature signature = new PdfSignature(document, document.Pages[0], null, "DigitalSignature");
            signature.ComputeHash += Signature_ComputeHash;

            //Save the PDF document.
            document.Save("ExternalSignature.pdf");

            //Close the document.
            document.Close(true);

            System.Diagnostics.Process.Start("ExternalSignature.pdf");
        }
        static void Signature_ComputeHash(object sender, PdfSignatureEventArgs arguments)
        {
            //Get the document bytes.
            byte[] documentBytes = arguments.Data;

            SignedCms signedCms = new SignedCms(new ContentInfo(documentBytes), detached: true);
            X509Store store = new X509Store("MY", StoreLocation.CurrentUser);
            store.Open(OpenFlags.ReadOnly | OpenFlags.OpenExistingOnly);
            X509Certificate2Collection collection = (X509Certificate2Collection)store.Certificates;
            //load your smart card certificate
            
            X509Certificate2 certificate = new X509Certificate2(collection[4]);
            CmsSigner cmsSigner = new CmsSigner(certificate);
            //Set the digest algorithm SHA256.
            cmsSigner.DigestAlgorithm = new Oid("2.16.840.1.101.3.4.2.1");
            signedCms.ComputeSignature(cmsSigner);
            //Embed the encoded digital signature to the PDF document.
            arguments.SignedData = signedCms.Encode();
        }
    }
}
